@@ -1,6 +1,6 @@
 ;;; parser-generator-lr-export.el --- Export LR(k) Parser -*- lexical-binding: t -*-
 
-;; Copyright (C) 2020-2021  Free Software Foundation, Inc.
+;; Copyright (C) 2020-2022  Free Software Foundation, Inc.
 
 
 ;;; Commentary:
@@ -13,7 +13,7 @@
 
 (defun parser-generator-lr-export-to-elisp (namespace &optional header copyright)
   "Export parser with NAMESPACE and a optional HEADER and COPYRIGHT."
-  (message "\nStarting generation of elips..\n")
+  (message "\n;; Starting generation of elips..\n")
 
   ;; Make sure all requisites are defined
   (unless parser-generator-lr--action-tables
@@ -1004,7 +1004,7 @@
        (buffer-substring-no-properties
         (point-min)
         (point-max))))
-    (message "\nCompleted generation of elips.\n")
+    (message "\n;; Completed generation of elips.\n")
     code))
 
 
